#/bin/bash

####################################################################################
### Цвета для подсказок
####################################################################################

#Black        0;30     Dark Gray     1;30
#Red          0;31     Light Red     1;31
#Green        0;32     Light Green   1;32
#Brown/Orange 0;33     Yellow        1;33
#Blue         0;34     Light Blue    1;34
#Purple       0;35     Light Purple  1;35
#Cyan         0;36     Light Cyan    1;36
#Light Gray   0;37     White         1;37

RED='\033[0;31m'
LIGHTRED='\033[1;35m'
GREEN='\033[0;32m'
YELLOW='\033[1;33m'
LIGHTGREEN='\033[1;32m'
DARKGRAY='\033[1;30m'
NC='\033[0m' # No Color

BOLD='\033[1m'
NORMAL='\033[0m'

####################################################################################
### Завершаем работу скрипта при ошибках
####################################################################################

set -eu

####################################################################################
### Задаём переменные
####################################################################################

# Переменные, настраиваемые пользователем, храним отдельно. 
# grep фильтрует переменные, исключая названия секций вроде "[FTP]"
source <(grep = /app/asr_billing/cfg/sorm3-mfi/config.ini)

# Пути к SQL файлам
host_sql_dir="${sql_dir}"
chroot_sql_dir="${sql_dir/\/app\/asr_billing/}"

# Папка для выгруженных файлов, сохраняем в историю
done_dir="${data_dir}/done/"

# Добавляем формат времени создания файлов согласно документации МФИ Софт 
file_date=$(date +%Y%m%d_%H%M)

# SQL, которые используем в этой выгрузке, передаём аргументами 
reports=( $@ )

####################################################################################
### Вспомогательные функции
####################################################################################

# Функция вычисления времени для лога.
print_date(){
    printf '%s %s' "$(date '+%Y-%m-%d %H:%M:%S')";
}

# Функция проверки, дебаг включен или нет
debug_status(){
    # Если среди аргументов есть "--debug" - возвращаем "1"
    if [[ "$@" =~ "--debug" ]]; then 
        return 1
    else 
        return 0
    fi 
}

####################################################################################
### Основные функции
####################################################################################

check(){
    # Проверяем что скрипту сказали какие выгрузки нужно делать
    # Если нет - даём подсказку и выходим
    if [[ ${#reports[@]} == "0" ]]; then 
        echo 
        echo "Укажите какие отчёты надо выгружать. Например:"
        echo 
        echo -e "  ${RED}upload.sh ${GREEN}abonent abonent_ident abonent_address abonent_service${NC}"
        echo 
        echo "Справочники и данные об абонентах, которые можно выгрузить этим скриптом:"
        echo -e "${LIGHTGREEN}"
        find ${sql_dir} -maxdepth 1 -type f -iname *sql -not -path '*template*' | sed "s|${sql_dir}/||g; s|.sql||g; s/^/  /g" | sort
        echo -e "${NC}"
        echo 
        echo "Ещё скрипт можно запустить с дебагом, тогда файлы будут созданы, но без выгрузки на FTP. Например:"
        echo 
        echo -e "  ${RED}upload.sh ${GREEN}abonent ${YELLOW}--debug${NC}"
        echo 
        exit 0

    fi 
    # Проверяем что есть директория данных, 
    # если нет - создаём вместе с поддиректорией done  
    if [[ ! -d ${data_dir}/done ]]; then 
        mkdir -p "${data_dir}/done"
    fi
    # Проверяем что указали параметры FTP.
    # Если нет - даём подсказку и выходим
    ftp_ip=${ftp_ip:-not_set}
    ftp_login=${ftp_login:-not_set}
    ftp_pass=${ftp_pass:-not_set}
    if [[ "$ftp_ip$ftp_login$ftp_pass" == *not_set* ]]; then 
        echo 
        echo "Укажите параметры FTP в файле /app/asr_billing/cfg/sorm3-mfi/config.ini"
        echo
        exit 0
    fi 
}

make_and_upload(){
    for report in ${reports[@]}; do 

        # Определяем пути к файлам
        raw_csv=${data_dir}/${report}.csv_raw
        heading_file=${sql_dir}/heading_${report}.csv
        # Название отчёта в имени файла всегда должно быть в верхнем регистре
        csv_to_upload=${data_dir}/${report^^}_${file_date}.csv

        # Выгружаем данные из базы
        # Условие на случай если справочник готовится как-то иначе
        # Например, pay_type.csv создаётся в configure.csv
        if [ -s $host_sql_dir/$report.sql ]; then 
            chroot /app/asr_billing/ isql-fb 169.254.30.50:/var/db/billing.gdb -p servicem -u SYSDBA -i $chroot_sql_dir/$report.sql | \
            sed '/^$/d'| \
            sed 's/ *$//g' | \
            iconv -f utf8 -t ${code_page} > $raw_csv
        elif [[ "${report}" =~ "--" ]]; then 
            break
        else 
            :
        fi 

        # Если выгрузка не пустая, обрабатываем
        if [ -f $raw_csv ] && [ -s $raw_csv ]; then
            print_date && echo "Файл $raw_csv, метеданные: $(ls -l ${raw_csv})"
            print_date && echo "Файл $raw_csv, кол. строк: $(cat ${raw_csv} | wc -l)"
            cat $heading_file $raw_csv > $csv_to_upload
            # Если включен дебаг - не выгружаем файл, а только выводим его название 
            if debug_status "${reports[@]}"; then 
                curl -v -sS --user $ftp_login:$ftp_pass -T ${csv_to_upload} ftp://$ftp_ip/ 
            else 
                print_date && echo "Пукть к файлу: $done_dir/${report^^}_${file_date}.csv"
            fi 
            mv ${csv_to_upload} $done_dir
            # На всякий случай, очищаем временный файл
            >$raw_csv
        elif [ ! -f $raw_csv ]; then 
            print_date && echo "SQL файла \"${report}.sql\" нет, возможно в имени файла ошибка"
        else 
            # Если выгрузка оказалась пустой - пишем об этом
            print_date && echo "Файл $raw_csv пуст"
        fi

    done 
}

####################################################################################
### Выполнение скрипта
####################################################################################

print_date && echo "скрипт запущен"

# Убедимся что есть всё необходимое для выгрузки, 
# выходим если исправить автоматом не получится
check
# Выгружаем 
make_and_upload

print_date && echo "скрипт завершен"